/* HighContrast theme variant */

@define-color phosh_fg_color black;
@define-color phosh_bg_color white;

@define-color phosh_borders_color alpha(@phosh_fg_color,.1);

@define-color phosh_activity_bg_color #e5e5e5;
@define-color phosh_notification_bg_color #e0e0e0;
@define-color phosh_splash_bg_color @theme_bg_color;
@define-color phosh_splash_fg_color @theme_fg_color;

/* Button Colors */
@define-color phosh_button_bg_color #e0e0e0;
@define-color phosh_button_hover_bg_color shade(@phosh_button_bg_color, 1.05);
@define-color phosh_button_active_bg_color shade(@phosh_button_bg_color, 1.1);

@define-color phosh_emergency_button_bg_color #e01b24;
@define-color phosh_emergency_button_fg_color #ffffff;

@import url("resource:///sm/puri/phosh/stylesheet/common.css");

#top-bar, #home-bar {
  box-shadow: inset 0 0 0 1px @phosh_borders_color;
}

.phosh-quick-setting,
button {
  box-shadow: inset 0 0 0 2px @phosh_borders_color;
}

button:hover {
  box-shadow: inset 0 0 0 2px @phosh_fg_color;
}

button:disabled {
  box-shadow: inset 0 0 0 2px alpha(@phosh_borders_color, 0.5);
}

.emergency-button:hover {
  box-shadow: inset 0 0 2px 0 shade(@phosh_emergency_button_bg_color, .6);
}
.emergency-button:focus {
  box-shadow: inset 0 0 2px 0 shade(@phosh_emergency_button_bg_color, .4);
}
