/*
 * Copyright (C) 2022 Purism SPC
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 */

#pragma once

#define EMERGENCY_INFO_GKEYFILE_LOCATION "phosh"
#define EMERGENCY_INFO_GKEYFILE_NAME     "EmergencyInfo.keyfile"
#define INFO_GROUP                       "Info"
#define CONTACTS_GROUP                   "Contacts"
